<?php

namespace Drupal\firebase_ui\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configuration form for Firebase UI settings.
 */
class FirebaseUISettingsForm extends ConfigFormBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a FirebaseUISettingsForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create($container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'firebase_ui_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['firebase_ui.settings'];
  }

  /**
   * Builds the settings form.
   *
   * @param array $form
   *   The form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The rendered form array.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('firebase_ui.settings');
    $form['apiKey'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#default_value' => $config->get('apiKey'),
      '#required' => TRUE,
      '#description' => $this->t('Enter your Firebase project\'s API key. You can find this in your Firebase Console under Project Settings. <a href="https://firebase.google.com/static/codelabs/firebase-get-to-know-web/img/f2c0f5838d0e8202_856.png" target="_blank">View instructions</a>.'),
    ];

    $form['projectId'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Project ID'),
      '#default_value' => $config->get('projectId'),
      '#required' => TRUE,
      '#description' => $this->t('Your Firebase project ID, <a href="https://firebase.google.com/static/codelabs/firebase-get-to-know-web/img/f2c0f5838d0e8202_856.png" target="_blank">View instructions</a>.'),
    ];

    $form['messagingSenderId'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Messaging Sender ID'),
      '#default_value' => $config->get('messagingSenderId'),
      '#required' => TRUE,
      '#description' => $this->t('This is the Sender ID used for Firebase Cloud Messaging (FCM). <a href="https://firebase.google.com/static/codelabs/firebase-get-to-know-web/img/f2c0f5838d0e8202_856.png" target="_blank">View instructions</a>.'),
    ];

    $form['appId'] = [
      '#type' => 'textfield',
      '#title' => $this->t('App ID'),
      '#default_value' => $config->get('appId'),
      '#required' => TRUE,
      '#description' => $this->t('Enter your Firebase App ID (e.g., <code>1:1234567890:web:abcdef123456</code>). <a href="https://firebase.google.com/static/codelabs/firebase-get-to-know-web/img/f2c0f5838d0e8202_856.png" target="_blank">View instructions</a>.'),
    ];

    $form['vapidKey'] = [
      '#type' => 'textfield',
      '#title' => $this->t('VAPID Key'),
      '#default_value' => $config->get('vapidKey'),
      '#required' => TRUE,
      '#description' => $this->t('Used for Web Push Protocol with Firebase. <a href="https://stackoverflow.com/a/54996207" target="_blank">View how to generate</a>.'),
    ];

    // Upload field remains unchanged:
    $form['service_account_file'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Firebase Admin SDK (JSON)'),
      '#upload_location' => 'private://firebase',
      '#default_value' => $config->get('service_account_file') ? [$config->get('service_account_file')] : NULL,
      '#upload_validators' => [
        'file_validate_extensions' => ['json'],
      ],
      '#description' => $this->t('
        Upload your Firebase Admin SDK JSON file, required for sending push notifications using Firebase Cloud Messaging. <br>
        You can generate this file from the <a href="https://medium.com/@naveennirbanyadav08/setup-gcp-service-account-for-fcm-9ec15e89fb74" target="_blank">Service Accounts</a> section in your Firebase Console. <br>
        The uploaded file will be stored securely in the private file system.
      '),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('firebase_ui.settings');

    foreach ([
      'apiKey',
      'authDomain',
      'projectId',
      'messagingSenderId',
      'measurementId',
      'vapidKey',
    ] as $field) {
      $config->set($field, $form_state->getValue($field));
    }

    // Save uploaded file reference using dependency injection.
    $fid = $form_state->getValue('service_account_file');
    if (!empty($fid[0])) {
      /** @var \Drupal\file\FileInterface $file */
      $file = $this->entityTypeManager->getStorage('file')->load($fid[0]);
      if ($file) {
        $file->setPermanent();
        $file->save();
        $config->set('service_account_file', $file->id());
      }
    }

    $config->save();
    parent::submitForm($form, $form_state);
  }

}
