<?php

namespace Drupal\firebase_ui\Plugin\QueueWorker;

use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\firebase_ui\Entity\FirebaseNotification;

/**
 * Processes items in the firebase_ui_notification_queue.
 *
 * @QueueWorker(
 *   id = "firebase_ui_notification_queue",
 *   title = @Translation("Firebase UI Notification Queue"),
 *   cron = {"time" = 30}
 * )
 */
class FirebaseNotificationQueueWorker extends QueueWorkerBase {

  /**
   * Processes a single queue item.
   *
   * @param mixed $data
   *   The queue data array containing 'nid'.
   */
  public function processItem($data) {
    if (empty($data['nid'])) {
      return;
    }

    $storage = \Drupal::entityTypeManager()->getStorage('firebase_notification');
    $notification = $storage->load($data['nid']);

    if (!$notification || !$notification->get('status')->value) {
      return;
    }

    $current_time = \Drupal::time()->getCurrentTime();
    $send_time = $notification->get('send_time')->value;

    if ($send_time > $current_time) {
      // Requeue for later.
      \Drupal::service('queue')->get('firebase_ui_notification_queue')->createItem([
        'nid' => $notification->id(),
      ]);
      return;
    }

    // Send to users.
    $users = $this->getTargetUsers($notification);
    $sender = \Drupal::service('firebase_ui.notification_sender');

    foreach ($users as $user) {
      // Loop through tokens for the user.
      $tokens = $user->get('field_firebase_tokens')->getValue();
      foreach ($tokens as $tokenItem) {
        $token = $tokenItem['value'];
        $sender->sendNotification(
          $token,
          $notification->get('title')->value,
          $notification->get('message')->value,
          [
            'type' => 'alert',
            'user_id' => $user->id(),
          ]
        );
      }
    }
  }

  /**
   * Gets all applicable users for the notification.
   *
   * @param \Drupal\firebase_ui\Entity\FirebaseNotification $notification
   *   The Firebase notification entity.
   *
   * @return \Drupal\user\UserInterface[]
   *   An array of loaded user entities.
   */
  private function getTargetUsers(FirebaseNotification $notification) {
    $type = $notification->get('type')->value;
    $target_ids = array_filter(explode(',', $notification->get('target_ids')->value));
    $uids = [];

    if ($type === 'all') {
      $all_uids = \Drupal::entityQuery('user')
        ->accessCheck(FALSE)
        ->condition('status', 1)
        ->execute();

      $users = \Drupal::entityTypeManager()->getStorage('user')->loadMultiple($all_uids);
      foreach ($users as $user) {
        if (!$user->get('field_firebase_tokens')->isEmpty()) {
          $uids[] = $user->id();
        }
      }
    }

    if ($type === 'specific_users') {
      $uids = $target_ids;
    }

    if ($type === 'roles') {
      $query = \Drupal::entityQuery('user')
        ->accessCheck(FALSE)
        ->condition('status', 1);
      $or = $query->orConditionGroup();
      foreach ($target_ids as $role) {
        $or->condition('roles', $role);
      }
      $uids = $query->condition($or)->execute();
    }

    return !empty($uids)
      ? \Drupal::entityTypeManager()->getStorage('user')->loadMultiple($uids)
      : [];
  }

}
